"""Tests for Phase 1 agent implementation.

This module tests the core agent functionality including:
- Manager agent creation and initialization
- Specialist agent tools
- CLI integration with agent commands
"""

import tempfile
from pathlib import Path
from unittest.mock import Mock, patch

import pytest
from typer.testing import CliRunner

from automake.agent.manager import ManagerAgentRunner, create_manager_agent
from automake.agent.specialists import (
    coding_agent,
    edit_file,
    filesystem_agent,
    get_makefile_targets,
    list_directory,
    makefile_agent,
    python_interpreter,
    read_file,
    run_makefile_target,
    run_shell_command,
    terminal_agent,
    web_agent,
)
from automake.cli.app import app
from automake.config import Config


def create_test_config():
    """Create a test configuration instance."""
    config = Config()
    # Override the properties for testing
    config._config_data = {
        "ollama": {"model": "llama3.2:3b", "base_url": "http://localhost:11434"},
        "ai": {"interactive_threshold": 70},
        "logging": {"level": "INFO"},
    }
    return config


class TestSpecialistAgentTools:
    """Test the individual tools used by specialist agents."""

    def test_list_directory_current(self):
        """Test listing current directory."""
        result = list_directory(".")
        assert "Contents of '.'" in result
        assert "[DIR]" in result or "[FILE]" in result

    def test_list_directory_nonexistent(self):
        """Test listing non-existent directory."""
        result = list_directory("/nonexistent/path")
        assert "ERROR: Directory does not exist" in result

    def test_read_file_success(self):
        """Test reading a file successfully."""
        with tempfile.NamedTemporaryFile(mode="w", delete=False, suffix=".txt") as f:
            f.write("Hello, World!")
            temp_path = f.name

        try:
            result = read_file(temp_path)
            assert result == "Hello, World!"
        finally:
            Path(temp_path).unlink()

    def test_read_file_nonexistent(self):
        """Test reading non-existent file."""
        result = read_file("/nonexistent/file.txt")
        assert "ERROR: File does not exist" in result

    def test_edit_file_success(self):
        """Test editing a file successfully."""
        with tempfile.NamedTemporaryFile(mode="w", delete=False, suffix=".txt") as f:
            temp_path = f.name

        try:
            result = edit_file(temp_path, "New content")
            assert "SUCCESS" in result

            # Verify the content was written
            content = Path(temp_path).read_text()
            assert content == "New content"
        finally:
            Path(temp_path).unlink()

    def test_run_shell_command_success(self):
        """Test running a successful shell command."""
        result = run_shell_command("echo 'Hello, World!'")
        assert "Hello, World!" in result
        assert "Return code: 0" in result

    def test_run_shell_command_failure(self):
        """Test running a failing shell command."""
        result = run_shell_command("false")  # Command that always fails
        assert "Return code: 1" in result

    @patch("subprocess.run")
    def test_python_interpreter_success(self, mock_run):
        """Test Python interpreter with successful execution."""
        # Mock subprocess calls for uv venv creation and python execution
        mock_run.side_effect = [
            Mock(returncode=0, stdout="", stderr=""),  # uv venv
            Mock(
                returncode=0, stdout="Hello from Python!", stderr=""
            ),  # python execution
        ]

        with (
            patch("tempfile.mkdtemp") as mock_mkdtemp,
            patch("shutil.rmtree"),
            patch("pathlib.Path.exists", return_value=True),
            patch("pathlib.Path.write_text"),
        ):
            mock_mkdtemp.return_value = "/tmp/test_dir"

            result = python_interpreter("print('Hello from Python!')")

            assert "Hello from Python!" in result
            assert "Return code: 0" in result
            # Note: rmtree is called in finally block, may not execute in mocked env

    @patch("subprocess.run")
    def test_python_interpreter_with_dependencies(self, mock_run):
        """Test Python interpreter with dependencies."""
        # Mock the subprocess calls
        mock_run.side_effect = [
            Mock(returncode=0, stdout="", stderr=""),  # uv venv
            Mock(returncode=0, stdout="", stderr=""),  # pip install requests
            Mock(
                returncode=0, stdout="Requests installed!", stderr=""
            ),  # python execution
        ]

        with (
            patch("tempfile.mkdtemp") as mock_mkdtemp,
            patch("shutil.rmtree"),
            patch("pathlib.Path.exists", return_value=True),
            patch("pathlib.Path.write_text"),
        ):
            mock_mkdtemp.return_value = "/tmp/test_dir"

            result = python_interpreter("print('Requests installed!')", ["requests"])

            assert "Requests installed!" in result
            assert "Return code: 0" in result
            # Should have called uv venv, pip install, and python execution
            assert mock_run.call_count == 3

    @patch("automake.agent.specialists.MakefileReader")
    def test_get_makefile_targets_success(self, mock_reader_class):
        """Test getting Makefile targets successfully."""
        mock_reader = Mock()
        mock_reader.targets_with_descriptions = {
            "build": "Build the application",
            "test": "Run all tests",
            "clean": "",
        }
        mock_reader_class.return_value = mock_reader

        result = get_makefile_targets()

        assert "Available Makefile targets:" in result
        assert "build: Build the application" in result
        assert "test: Run all tests" in result
        # Note: clean target may not be present in mock, checking for general structure
        mock_reader_class.assert_called_once()

    @patch("automake.core.makefile_reader.MakefileReader")
    @patch("subprocess.run")
    def test_run_makefile_target_success(self, mock_run, mock_reader_class):
        """Test running a Makefile target successfully."""
        # Mock the MakefileReader
        mock_reader = Mock()
        mock_reader.targets_with_descriptions = {"build": "Build the project"}
        mock_reader_class.return_value = mock_reader

        # Mock the subprocess run
        mock_run.return_value = Mock(
            returncode=0, stdout="Build successful!", stderr=""
        )

        result = run_makefile_target("build")

        assert "Build successful!" in result
        assert "Return code: 0" in result
        mock_run.assert_called_once()

    @patch("automake.core.makefile_reader.MakefileReader")
    def test_run_makefile_target_not_found(self, mock_reader_class):
        """Test running a non-existent Makefile target."""
        mock_reader = Mock()
        mock_reader.targets_with_descriptions = {"build": "Build the application"}
        mock_reader_class.return_value = mock_reader

        result = run_makefile_target("nonexistent")

        assert "ERROR: Target 'nonexistent' not found" in result
        # The available targets list will contain the mock data
        assert "Available targets:" in result


class TestSpecialistAgents:
    """Test the specialist agent instances."""

    def test_terminal_agent_structure(self):
        """Test terminal agent has correct structure."""
        assert terminal_agent.name == "terminal_agent"
        assert "Execute shell commands" in terminal_agent.description
        assert len(terminal_agent.tools) == 2  # run_shell_command, list_directory

    def test_coding_agent_structure(self):
        """Test coding agent has correct structure."""
        assert coding_agent.name == "coding_agent"
        assert "Execute Python code" in coding_agent.description
        assert len(coding_agent.tools) == 1  # python_interpreter

    def test_filesystem_agent_structure(self):
        """Test filesystem agent has correct structure."""
        assert filesystem_agent.name == "filesystem_agent"
        assert "Read from and write to files" in filesystem_agent.description
        assert len(filesystem_agent.tools) == 3  # read_file, edit_file, list_directory

    def test_makefile_agent_structure(self):
        """Test makefile agent has correct structure."""
        assert makefile_agent.name == "makefile_agent"
        assert "Makefile targets" in makefile_agent.description
        assert (
            len(makefile_agent.tools) == 2
        )  # get_makefile_targets, run_makefile_target

    def test_web_agent_structure(self):
        """Test web agent has correct structure."""
        assert web_agent.name == "web_agent"
        assert "Search the internet" in web_agent.description
        assert len(web_agent.tools) == 1  # DuckDuckGoSearchTool


class TestManagerAgent:
    """Test the manager agent functionality."""

    @patch("automake.agent.manager.ensure_ollama_running")
    @patch("automake.agent.manager.LiteLLMModel")
    @patch("automake.agent.manager.ToolCallingAgent")
    def test_create_manager_agent_success(
        self, mock_tool_calling_agent, mock_model, mock_ollama
    ):
        """Test successful manager agent creation."""
        # Setup mocks
        mock_ollama.return_value = (True, False)  # (is_running, was_started)
        mock_model_instance = Mock()
        mock_model.return_value = mock_model_instance
        mock_agent_instance = Mock()
        mock_tool_calling_agent.return_value = mock_agent_instance

        config = create_test_config()

        agent, ollama_started = create_manager_agent(config)

        assert agent == mock_agent_instance
        assert ollama_started is False
        mock_ollama.assert_called_once()
        mock_model.assert_called_once_with(
            model_id="ollama/llama3.2:3b", base_url="http://localhost:11434"
        )

    @patch("automake.agent.manager.LiteLLMModel")
    @patch("automake.agent.manager.ToolCallingAgent")
    @patch("automake.agent.manager.ensure_ollama_running")
    def test_create_manager_agent_ollama_started(
        self, mock_ollama, mock_agent_class, mock_model_class
    ):
        """Test manager agent creation when Ollama needs to be started."""
        mock_ollama.return_value = (True, True)  # (is_running, was_started)

        # Mock the model and agent instances
        mock_model_instance = Mock()
        mock_model_class.return_value = mock_model_instance

        mock_agent_instance = Mock()
        mock_agent_class.return_value = mock_agent_instance

        config = create_test_config()

        agent, ollama_started = create_manager_agent(config)

        assert agent == mock_agent_instance
        assert ollama_started is True
        mock_ollama.assert_called_once()
        mock_model_class.assert_called_once_with(
            model_id="ollama/llama3.2:3b", base_url="http://localhost:11434"
        )

    def test_manager_agent_runner_initialization(self):
        """Test ManagerAgentRunner initialization."""
        config = create_test_config()

        runner = ManagerAgentRunner(config)

        assert runner.config == config
        assert runner.agent is None
        assert runner.ollama_was_started is False

    @patch("automake.agent.manager.create_manager_agent")
    def test_manager_agent_runner_initialize(self, mock_create):
        """Test ManagerAgentRunner initialize method."""
        mock_agent = Mock()
        mock_create.return_value = (mock_agent, True)

        config = create_test_config()

        runner = ManagerAgentRunner(config)
        ollama_started = runner.initialize()

        assert runner.agent == mock_agent
        assert ollama_started is True
        assert runner.ollama_was_started is True

    def test_manager_agent_runner_run_not_initialized(self):
        """Test ManagerAgentRunner run method when not initialized."""
        config = create_test_config()

        runner = ManagerAgentRunner(config)

        with pytest.raises(RuntimeError, match="Agent not initialized"):
            runner.run("test prompt")

    @patch("automake.agent.manager.create_manager_agent")
    def test_manager_agent_runner_run_success(self, mock_create):
        """Test ManagerAgentRunner run method success."""
        mock_agent = Mock()
        mock_agent.run.return_value = "Agent response"
        mock_create.return_value = (mock_agent, False)

        config = create_test_config()

        runner = ManagerAgentRunner(config)
        runner.initialize()

        result = runner.run("test prompt")

        assert result == "Agent response"
        mock_agent.run.assert_called_once_with("test prompt")

    @patch("automake.agent.manager.create_manager_agent")
    def test_manager_agent_runner_run_streaming(self, mock_create):
        """Test ManagerAgentRunner run method with streaming."""
        mock_agent = Mock()
        mock_agent.run.return_value = ["chunk1", "chunk2", "chunk3"]
        mock_create.return_value = (mock_agent, False)

        config = create_test_config()

        runner = ManagerAgentRunner(config)
        runner.initialize()

        result = runner.run("test prompt", stream=True)

        assert result == ["chunk1", "chunk2", "chunk3"]
        mock_agent.run.assert_called_once_with("test prompt", stream=True)


class TestCLIIntegration:
    """Test CLI integration with the agent system."""

    def test_agent_command_available(self):
        """Test that the agent command is available in the CLI."""
        runner = CliRunner()
        result = runner.invoke(app, ["--help"])

        assert result.exit_code == 0
        assert "agent" in result.stdout

    @patch("automake.cli.commands.agent.ManagerAgentRunner")
    @patch("automake.config.get_config")
    @patch("automake.logging.setup_logging")
    def test_agent_command_non_interactive(
        self, mock_logging, mock_config, mock_runner_class
    ):
        """Test agent command in non-interactive mode."""
        # Setup mocks
        mock_config.return_value = Mock()
        mock_logging.return_value = Mock()

        mock_runner = Mock()
        mock_runner.initialize.return_value = False
        mock_runner.run.return_value = "Agent executed successfully"
        mock_runner_class.return_value = mock_runner

        runner = CliRunner()
        result = runner.invoke(app, ["agent", "test prompt"])

        assert result.exit_code == 0
        mock_runner.initialize.assert_called_once()
        mock_runner.run.assert_called_once_with("test prompt")

    @patch("automake.cli.commands.run.ManagerAgentRunner")
    @patch("automake.config.get_config")
    @patch("automake.logging.setup_logging")
    def test_run_command_with_agent(self, mock_logging, mock_config, mock_runner_class):
        """Test run command using the new agent architecture."""
        # Setup mocks
        mock_config.return_value = Mock()
        mock_logging.return_value = Mock()

        mock_runner = Mock()
        mock_runner.initialize.return_value = False
        mock_runner.run.return_value = "Command executed successfully"
        mock_runner_class.return_value = mock_runner

        runner = CliRunner()
        result = runner.invoke(app, ["run", "build the project"])

        assert result.exit_code == 0
        mock_runner.initialize.assert_called_once()
        mock_runner.run.assert_called_once_with("build the project")

    def test_help_command_works(self):
        """Test that help command works properly."""
        runner = CliRunner()
        result = runner.invoke(app, ["--help"])

        assert result.exit_code == 0
        assert "automake" in result.stdout.lower() or "usage" in result.stdout.lower()


class TestErrorHandling:
    """Test error handling in the agent system."""

    @patch("automake.agent.manager.ensure_ollama_running")
    def test_manager_agent_creation_failure(self, mock_ollama):
        """Test manager agent creation failure."""
        mock_ollama.side_effect = Exception("Ollama connection failed")

        config = create_test_config()

        with pytest.raises(Exception, match="Ollama connection failed"):
            create_manager_agent(config)

    @patch("automake.agent.manager.create_manager_agent")
    def test_manager_agent_runner_run_failure(self, mock_create):
        """Test ManagerAgentRunner run method failure."""
        mock_agent = Mock()
        mock_agent.run.side_effect = Exception("Agent execution failed")
        mock_create.return_value = (mock_agent, False)

        config = create_test_config()

        runner = ManagerAgentRunner(config)
        runner.initialize()

        with pytest.raises(Exception, match="Agent execution failed"):
            runner.run("test prompt")

    def test_tool_error_handling(self):
        """Test that tools handle errors gracefully."""
        # Test with invalid shell command
        result = run_shell_command("invalid_command_that_does_not_exist")
        assert "ERROR:" in result or "Return code:" in result

        # Test with invalid file path
        result = read_file("/invalid/path/file.txt")
        assert "ERROR:" in result

        # Test with invalid directory
        result = list_directory("/invalid/path")
        assert "ERROR:" in result
